#![allow(async_fn_in_trait)]

pub mod fs;
pub mod rentry;

use std::io::Result;

use crate::FileDescriptor;

/// An engine that drives remote storage.
pub trait Engine {
    const CHUNK_SIZE: usize;

    /// Returns authentication details (CSRF token, session token, etc).
    async fn auth(&mut self) -> Result<()>;
    /// Processes a file and returns a [`FileDescriptor`].
    async fn process(&self, name: String, data: Vec<u8>) -> Result<FileDescriptor>;
    /// Reads a [`FileDescriptor`] and recreates the file.
    async fn reconstruct(&self, descriptor: FileDescriptor) -> Result<()>;
    /// Reads a [`FileDescriptor`] and deletes the file.
    async fn delete(&mut self, descriptor: FileDescriptor) -> Result<()>;
    /// Gets the data in a chunk by its ID.
    async fn get_chunk(&self, id: &str) -> Result<String>;
    /// Creates a new chunk and sends it to remote.
    async fn create_chunk(&self, id: &str, data: String) -> Result<String>;
    /// Deletes a specific chunk from the remote. Used to clean up remote data
    /// when deleting a file.
    async fn delete_chunk(&self, id: &str) -> Result<()>;
}
